#pragma once

#include "NPSingleton.h"
#include "NPUnzipFile.h"
#include "NBinaryResourceLoader.h"
#include "NLayoutResourceLoader.h"
#include "NStringResourceLoader.h"
#include "AbstractModel.h"
#include "AbstractView.h"
#include "AbstractControl.h"
#include <GdiPlus.h>
#include "NPMemoryData.h"

interface NPCOMMONUI_API IMVCFactory
{
public:
	virtual ~IMVCFactory() {}

	virtual AbstractModel* CreateModel() = 0;
	virtual AbstractView* CreateView() = 0;
	virtual AbstractControl* CreateControl() = 0;

	virtual void DestroyModel(AbstractModel* model) = 0;
	virtual void DestroyView(AbstractView* view) = 0;
	virtual void DestroyControl(AbstractControl* control) = 0;

	virtual BOOL GetResourcePath(TCHAR* path, int lenPath) = 0;
};

class NPCOMMONUI_API Application
{
public:
	DECLARE_SINGLETON_CODE(Application);
private:
	Application(void);
	~Application(void) {}

public:
	static LPCTSTR LoadString(LPCTSTR pszMsgID);
	static IImage* LoadImage(LPCTSTR pszcImageID);
	static BOOL FreeImage(IImage* pImage);
	static TiXmlElement* LoadLayoutNode(LPCTSTR pszContainerID);

	void Init(IMVCFactory* facotory);
	void Close();
	void PreRun();
	void Run();
	void PostRun();

	AbstractModel* GetModel() { return m_Model; }
	AbstractView* GetView() { return m_View; }
	AbstractControl* GetControl() { return m_Control; }

	CNBinaryResourceLoader* GetBinaryLoader() { return &m_DefaultBinaryLoader; }
	CNLayoutResourceLoader* GetLayoutLoader() { return &m_DefaultLayoutLoader; }
	CNStringResourceLoader* GetStringLoader() { return &m_DefaultStringLoader; }

private:
	BOOL InitReousrce(IMVCFactory* facotory);
	BOOL CloseReousrce();

private:
	IMVCFactory* m_MVCFactory;
	AbstractModel* m_Model;
	AbstractView* m_View;
	AbstractControl* m_Control;

	CNPUnzipFile	m_ZipImage;
	CNPMemoryData*  m_ZipMemory;
	CNBinaryResourceLoader m_DefaultBinaryLoader;
	CNLayoutResourceLoader m_DefaultLayoutLoader;
	CNStringResourceLoader m_DefaultStringLoader;

	Gdiplus::GdiplusStartupInput gdiPlusStartupInput;
	ULONG_PTR gdiplusToken;
	Gdiplus::GdiplusStartupOutput gdiPlusStartupOutput;
};
